import json

from .path import abs_path
from .files import verify_file

class SrbJson:
    '''
    A json class with common functionality.
    parameters:
        file_path - name/path of file
        template  - a basic template for file.
                    can be of two forms:
                        1. {}   # default
                        2. { 'app_name':{} } # default embeded in something, helps to check if file is good/bad
        strict - strict json follows template strictly and user can only modify values not keys
    '''
    def __init__(self,file_path,template={},strict=False):
        self.strict = strict
        self.file_path = abs_path(file_path)
        self.template = template
        self.masterkey = SrbJson._get_master_key(template) # generlly 'app_name'
        self.fetch_data()

    srblib_template = {
        "srblib": # masterkey
        {
            "debug":False,
        }
    }

    def fetch_data(self):
        self.data = SrbJson.extract_data(self.file_path,self.template) # also creates empty if not there
        if self.strict:
            template = self.template
            if(self.masterkey): template = template[self.masterkey]
            for key in template.keys():
                if not key in self.data:
                    raise Exception('key' +key+ ' missing, json should be similar to template in strict mode')
            for key in self.data:
                if not key in template:
                    raise Exception('Extra key present ' +key+ ', json should be similar to template in strict mode')

        return self.data

    def keys(self):
        return self.data.keys()

    def __getitem__(self,index):
        self.fetch_data()
        return self.data[index]

    def __setitem__(self,index,value):
        if self.strict:
            if index in self.data:
                self.data[index]=value
            else:
                raise Exception('cannot add new key ' +index+ ' in strict mode')
        else:
            self.data[index]=value
        self._burn_data_to_file()

    def __contains__(self,value):
        return value in self.data

    def __delitem__(self,index):
        '''
        deletion is not allowed in strict mode. just modification is permitted
        better to set value to None
        to preserve template structure
        '''
        if self.strict:
            self[index] = None
            return

        del self.data[index]
        data = self.data
        if(self.masterkey):
            data = {self.masterkey:data}

        jfile = open(self.file_path, 'w')
        json.dump(data,jfile,indent = 4,ensure_ascii = False)
        jfile.close()


    @staticmethod
    def extract_data(file_path,template={}):
        """
        Extracts json data from the given file
        if there is no such file
            it will create one
        if there is currupt file (only for srb standard templates)
            it will create new
        if file is ok
            it will return its content
        """
        fille = abs_path(file_path)
        try:
            jfile = open(fille)
        except FileNotFoundError:
            SrbJson._create_file(fille,template)
        jfile = open(fille)
        data = json.load(jfile)

        masterkey = SrbJson._get_master_key(template) # if template is in srb standard
        if(masterkey):
            if(not masterkey in data.keys()):
                SrbJson._create_file(fille,template)
                jfile = open(fille)
                data = json.load(jfile)
            return data[masterkey]
        else:
            return data

    def _burn_data_to_file(self):
        """
        burns data to original file
        it is insecure way should be avoided to be directly used by user

        """
        data = self.data
        if(self.masterkey):
            data = {self.masterkey:data}

        jfile = open(self.file_path, 'w')
        json.dump(data,jfile,indent = 4,ensure_ascii = False)
        jfile.close()


    @staticmethod
    def dump_data(data,file_path,template):
        """
        adds data to original file
        it is bit secure, doesn't add extra key to data if not present in jfile already
        """
        fille = abs_path(file_path)
        dictt = SrbJson.extract_data(fille,template)
        for key in data:
            if(key in dictt):
                dictt[key] = data[key]

        masterkey = SrbJson._get_master_key(template) # just to avoid passing extra parameter as masterkey
        if(masterkey):
            dictt = {masterkey:dictt}

        jfile = open(abs_path(file_path), 'w')
        json.dump(dictt,jfile,indent = 4,ensure_ascii = False)
        jfile.close()



    @staticmethod
    def _create_file(fille,template):
        verify_file(fille)
        jfile = open(fille, 'w')
        json.dump(template,jfile,indent = 4,ensure_ascii = False)
        jfile.close()

    @staticmethod
    def _get_master_key(template):
        keys = template.keys() # if template is in srb standard
        if(len(keys) == 1 and type(template[list(keys)[0]]) == dict):
            return list(keys)[0]
        else:
            return None
